#
# AWS v4 credential configuration (v1.0)
#
#  https://docs.aws.amazon.com/cli/latest/userguide/cli-configure-files.html
#
# aws_credentials fields:
#
#  * aws_access_key_id
#  * aws_secret_access_key
#
# aws_config fields:
#
#  * region
#

import kvstore;

sub vcl_init
{
	# AWS configuration files
	new aws_credentials = kvstore.init();
	new aws_config = kvstore.init();

	# Request signing
	new aws_sign = kvstore.init(scope = REQUEST);
}

# Init signature values
sub aws_sign_init
{
	# Profile detection (_prefix)
	if (aws_sign.get("profile")) {
		aws_sign.set("_prefix", aws_sign.get("profile") + "_");
	} else if (aws_config.get("profile")) {
		aws_sign.set("_prefix", aws_config.get("profile") + "_");
	} else if (aws_credentials.get("default_aws_access_key_id")) {
		aws_sign.set("_prefix", "default_");
	} else if (aws_config.get("default_aws_access_key_id")) {
		aws_sign.set("_prefix", "default_");
	} else {
		aws_sign.delete("_prefix");
	}

	# aws_access_key_id (_access_key)
	if (aws_credentials.get(aws_sign.get("_prefix") + "aws_access_key_id")) {
		aws_sign.set("_access_key", aws_credentials.get(aws_sign.get("_prefix") + "aws_access_key_id"));
	} else if (aws_config.get(aws_sign.get("_prefix") + "aws_access_key_id")) {
		aws_sign.set("_access_key", aws_config.get(aws_sign.get("_prefix") + "aws_access_key_id"));
	} else if (aws_config.get("profile " + aws_sign.get("_prefix") + "aws_access_key_id")) {
		aws_sign.set("_access_key", aws_config.get("profile " + aws_sign.get("_prefix") + "aws_access_key_id"));
	} else {
		call aws_sign_finish;
		return (fail("AWS configuration not found. Required: aws_access_key_id"));
	}

	# aws_secret_access_key (_secret_key)
	if (aws_credentials.get(aws_sign.get("_prefix") + "aws_secret_access_key")) {
		aws_sign.set("_secret_key", aws_credentials.get(aws_sign.get("_prefix") + "aws_secret_access_key"));
	} else if (aws_config.get(aws_sign.get("_prefix") + "aws_secret_access_key")) {
		aws_sign.set("_secret_key", aws_config.get(aws_sign.get("_prefix") + "aws_secret_access_key"));
	} else if (aws_config.get("profile " + aws_sign.get("_prefix") + "aws_secret_access_key")) {
		aws_sign.set("_secret_key", aws_config.get("profile " + aws_sign.get("_prefix") + "aws_secret_access_key"));
	} else if (aws_credentials.get("default_aws_secret_access_key")) {
		aws_sign.set("_secret_key", aws_credentials.get("default_aws_secret_access_key"));
	} else {
		call aws_sign_finish;
		return (fail("AWS configuration not found. Required: aws_secret_access_key"));
	}

	# HMAC algorithm (_algorithm) that was used to produce the secret_key
	if (aws_credentials.get(aws_sign.get("_prefix") + "algorithm")) {
		aws_sign.set("_algorithm", aws_credentials.get(aws_sign.get("_prefix") + "algorithm"));
	} else if (aws_config.get(aws_sign.get("_prefix") + "algorithm")) {
		aws_sign.set("_algorithm", aws_config.get(aws_sign.get("_prefix") + "algorithm"));
	} else if (aws_config.get("profile " + aws_sign.get("_prefix") + "algorithm")) {
		aws_sign.set("_algorithm", aws_config.get("profile " + aws_sign.get("_prefix") + "algorithm"));
	} else if (aws_config.get("default_algorithm")) {
		aws_sign.set("_algorithm", aws_config.get("default_algorithm"));
	} else {
		aws_sign.set("_algorithm", "AWS4-HMAC-SHA256");
	}

	# region (_region)
	if (aws_config.get(aws_sign.get("_prefix") + "region")) {
		aws_sign.set("_region", aws_config.get(aws_sign.get("_prefix") + "region"));
	} else if (aws_config.get("profile " + aws_sign.get("_prefix") + "region")) {
		aws_sign.set("_region", aws_config.get("profile " + aws_sign.get("_prefix") + "region"));
	} else if (aws_config.get("default_region")) {
		aws_sign.set("_region", aws_config.get("default_region"));
	} else {
		call aws_sign_finish;
		return (fail("AWS configuration not found. Required: region"));
	}
}

# Cleanup signature values
sub aws_sign_finish
{
	aws_sign.init(scrub = true);
}

# EOF
