#
# AWS v4 request signing (v1.1)
#
# aws_sign fields:
#
#  * profile (optional)
#  * service (optional, default: s3)
#  * header_regex (optional) - regex capturing extra headers to sign
#

import crypto;
import goto;
import headerplus;
import kvstore;
import std;
import urlplus;
import utils;
import xbody;

sub vcl_init
{
	# From init
	#new aws_sign = kvstore.init(scope = REQUEST);
}

# Grab the bereq.body hash (optional)
sub aws_sign_req_body
{
	if (!req.http.x-amz-content-sha256) {
		set req.http.x-amz-content-sha256 = crypto.hex_encode(xbody.get_req_body_hash(sha256));
	}
}

# Sign the bereq
sub aws_sign_bereq
{
	# Setup
	call aws_sign_init;
	urlplus.reset();

	# Start signature
	set bereq.http.x-amz-date = utils.time_format("%Y%m%dT%H%M%SZ");
	aws_sign.set("_datestamp", utils.time_format("%Y%m%d"));

	if (aws_sign.get("debug_aws_sign", "0") == "1") {
		set bereq.http.x-amz-date = aws_sign.get("debug_aws_sign_x-amz-date");
		aws_sign.set("_datestamp", aws_sign.get("debug_aws_sign_datestamp"));
	}

	if (!bereq.http.x-amz-content-sha256) {
		set bereq.http.x-amz-content-sha256 = crypto.hex_encode(crypto.hash(sha256, ""));
	}

	headerplus.init(bereq);
	headerplus.keep("host");
	#headerplus.keep("range");
	headerplus.keep_regex("^x-amz-");
	if (aws_sign.get("header_regex")) {
		headerplus.keep_regex(aws_sign.get("header_regex"));
	}

	aws_sign.set("_signed_headers", headerplus.as_list(NAME, ";", name_case = LOWER));
	aws_sign.set("_scope", aws_sign.get("_datestamp") + "/" +
		aws_sign.get("_region") + "/" +
		aws_sign.get("service", "s3") +
		"/aws4_request");

	aws_sign.set("_canonical_request",
		crypto.hex_encode(crypto.hash(sha256,
			bereq.method + utils.newline() +
			urlplus.url_as_string() + utils.newline() +
			urlplus.query_as_string(query_keep_equal_sign=1) + utils.newline() +
			headerplus.as_list(BOTH, utils.newline(), ":", name_case = LOWER) + utils.newline() +
			utils.newline() +
			aws_sign.get("_signed_headers") + utils.newline() +
			bereq.http.x-amz-content-sha256)
		));

	aws_sign.set("_signature",
		crypto.hex_encode(crypto.hmac(sha256,
			crypto.hmac(sha256,
			crypto.hmac(sha256,
			crypto.hmac(sha256,
			crypto.hmac(sha256,
			crypto.blob("AWS4" + aws_sign.get("_secret_key")),
				aws_sign.get("_datestamp")),
				aws_sign.get("_region")),
				aws_sign.get("service", "s3")),
				"aws4_request"),
					aws_sign.get("_algorithm") + utils.newline() +
					bereq.http.x-amz-date + utils.newline() +
					aws_sign.get("_scope") + utils.newline() +
					aws_sign.get("_canonical_request"))
		));

	set bereq.http.Authorization =
		aws_sign.get("_algorithm") + " " +
		"Credential=" + aws_sign.get("_access_key") + "/" +
		aws_sign.get("_scope") + ", " +
		"SignedHeaders=" + aws_sign.get("_signed_headers") + ", " +
		"Signature=" + aws_sign.get("_signature");

	headerplus.reset();
	call aws_sign_finish;
}

# Ignore vcc unused warning for aws_sign_req_body
sub vcl_recv
{
	if (0) {
		call aws_sign_req_body;
	}
}

#EOF
