#
# Probe Proxy v1.1
#

import kvstore;
import probe_proxy;
import std;
import str;
import udo;

sub vcl_init {
	# Settings
	new probe_proxy_opts = kvstore.init();

	# probe_proxy_opts.set("per_host", "false");
	# probe_proxy_opts.set("retries", "0");
	# probe_proxy_opts.set("call_recv", "false");
	# probe_proxy_opts.set("call_backend_response", "false");
	# probe_proxy_opts.set("call_deliver", "false");

	# Always route to Varnish
	probe_proxy.global_override(probe_proxy.self());

	# When used all probes will be forwarded to this gateway
	# Parent VCL fills this in with probe gateway backends
	new probe_proxy_gateway = udo.director();
	probe_proxy_gateway.set_type(fallback);
}

sub vcl_recv {
	if (probe_proxy.is_probe()) {
		if (str.contains(req.http.VPP-path, server.identity)) {
			return (synth(508));
		}

		set req.http.VPP-path = server.identity + " " + req.http.VPP-path;

		if (probe_proxy_opts.get("call_recv") != "true") {
			return (hash);
		}
	}
}

sub vcl_hash {
	if (probe_proxy.is_probe()) {
		hash_data(req.url);
		if (probe_proxy_opts.get("per_host", "false") == "true") {
			hash_data(req.http.Host);
		} else {
			hash_data(probe_proxy.backend());
		}
		return (lookup);
	}
}

sub vcl_backend_fetch {
	if (probe_proxy.is_probe()) {
		unset bereq.http.VPP-retry;
		# Use the probe gateway (All probes to 1 backend)
		if (std.healthy(probe_proxy_gateway.backend()) &&
		    !bereq.http.VPP-mark) {
			set bereq.backend = probe_proxy_gateway.backend();

			# Dont loop around
			set bereq.http.VPP-mark = "true";
			set bereq.http.VPP-retry = "true";
		} else {
			# We are a real probe, go to the probe backend (1 probe to 1 backend)
			set bereq.backend = probe_proxy.backend();
			# Make sure we close old connection and get a new one
			probe_proxy.force_fresh();
		}
		# We need to always go to a backend because we are the probe
		probe_proxy.skip_health_check();
		set bereq.first_byte_timeout = probe_proxy.timeout();
		set bereq.connect_timeout = probe_proxy.timeout();

		return (fetch);
	}
}

sub vcl_backend_response {
	if (probe_proxy.is_probe()) {
		if (bereq.http.VPP-retry && beresp.http.VPP-looped && beresp.status == 508) {
			return (error(508));
		}
		set beresp.ttl = probe_proxy.interval();
		set beresp.grace = 0s;
		set beresp.keep = 0s;
		if (probe_proxy_opts.get("call_backend_response") != "true") {
			return (deliver);
		}
	}
}

sub vcl_backend_error {
	if (probe_proxy.is_probe()) {
		if (bereq.http.VPP-retry) {
			if (bereq.retries < std.integer(probe_proxy_opts.get("retries"), 0)) {
				unset bereq.http.VPP-mark;
			}
			return (retry);
		}

		set beresp.ttl = probe_proxy.interval();
		set beresp.grace = 0s;
		set beresp.keep = 0s;
		return (deliver);
	}
}

sub vcl_deliver {
	if (probe_proxy.is_probe()) {
		if (probe_proxy_opts.get("call_deliver") != "true") {
			return (deliver);
		}
	}
}

sub vcl_synth {
	if (probe_proxy.is_probe()) {
		if (resp.status == 508) {
			set resp.http.VPP-looped = "true";
		}
		return (deliver);
	}
}

# EOF
