/*
 * Copyright (c) 2020 Varnish Software AS
 * All rights reserved.
 *
 * Author: Alve Elde <alve@varnish-software.com>
 *
 */

import rewrite;
import kvstore;
import resolver;

sub vcl_init {
	new vb_domains = kvstore.init();
	new vb_ua_filter = rewrite.ruleset(string = {""}, type = regex, min_fields = 1);
	new vb_domain_rules = rewrite.ruleset(string = {""}, type = suffix, min_fields = 2);
}

sub vb_check_client {
	unset req.http.vb-error;
	unset req.http.vb-access;
	unset req.http.vb-domain;

	// Filter out irrelevant clients based on User-Agent
	if (!vb_ua_filter.match(req.http.User-Agent)) {
		set req.http.vb-error = "VB: Client UA did not pass UA filter";
		return;
	}

	// Check for a previous resolution of the client ip in cache
	set req.http.vb-domain = vb_domains.get(client.ip, "");

	// If not in cache, resolve the client domain name and cache it
	if (req.http.vb-domain == "") {
		if (!resolver.resolve()) {
			unset req.http.vb-domain;
			set req.http.vb-error = "VB: Resolve failed with: " + resolver.error();
			return;
		}
		set req.http.vb-domain = resolver.domain();
		vb_domains.set(client.ip, req.http.vb-domain);
	}

	// Suffix-match the the client domain name
	if(!vb_domain_rules.match(req.http.vb-domain)) {
		set req.http.vb-error = "VB: Client domain did not match any rule";
		return;
	}

	set req.http.vb-access = vb_domain_rules.rewrite(2, mode = only_matching);
}
